@@ -43,7 +43,7 @@ public:
         DOMDocument* doc=XMLToolingConfig::getConfig().getParser().parse(in);
         XercesJanitor<DOMDocument> janitor(doc);
         m_resolver = XMLToolingConfig::getConfig().CredentialResolverManager.newPlugin(
-            FILESYSTEM_CREDENTIAL_RESOLVER,doc->getDocumentElement()
+            CHAINING_CREDENTIAL_RESOLVER,doc->getDocumentElement()
             );
         XMLObjectBuilder::registerDefaultBuilder(new UnknownElementBuilder());
     }
